from ..utils import StrEnum


class TipoOperacion(StrEnum):
    PRESTACION_DE_SERVICIOS_PROFESIONALES = "03"
    ARRENDAMIENTO_DE_INMUEBLES = "06"
    OTROS = "85"


class TipoTercero(StrEnum):
    PROVEEDOR_NACIONAL = "04"
    PROVEEDOR_EXTRANJERO = "05"
    PROVEEDOR_GLOBAL = "15"


class Periodo(StrEnum):
    ENERO = "01"
    FEBRERO = "02"
    MARZO = "03"
    ABRIL = "04"
    MAYO = "05"
    JUNIO = "06"
    JULIO = "07"
    AGOSTO = "08"
    SEPTIEMBRE = "09"
    OCTUBRE = "10"
    NOVIEMBRE = "11"
    DICIEMBRE = "12"

    ENERO_FEBRERO = "13"
    MARZO_ABRIL = "14"
    MAYO_JUNIO = "15"
    JULIO_AGOSTO = "16"
    SEPTIEMBRE_OCTUBRE = "17"
    NOVIEMBRE_DICIEMBRE = "18"

    ENERO_MARZO = "19"
    ABRIL_JUNIO = "20"
    JULIO_SEPTIEMBRE = "21"
    OCTUBRE_DICIEMBRE = "22"

    ENERO_ABRIL = "23"
    MAYO_AGOSTO = "24"
    SEPTIEMBRE_DICIEMBRE = "25"

    ENERO_JUNIO = "26"
    JULIO_DICIEMBRE = "27"


class Pais(StrEnum):
    PRINCIPADO_DE_ANDORRA = 'AD'
    EMIRATOS_ARABES_UNIDOS = 'AE'
    AFGANISTAN = 'AF'
    ANTIGUA_Y_BERMUDA = 'AG'
    ISLA_ANGUILLA = 'AI'
    REPUBLICA_DE_ALBANIA = 'AL'
    ANTILLAS_NEERLANDESAS = 'AN'
    REPUBLICA_DE_ANGOLA = 'AO'
    ANTARTICA = 'AQ'
    ARGENTINA = 'AR'
    SAMOA_AMERICANA = 'AS'
    AUSTRIA = 'AT'
    AUSTRALIA = 'AU'
    ARUBA = 'AW'
    ASCENSION = 'AX'
    ISLAS_AZORES = 'AZ'
    BARBADOS = 'BB'
    BANGLADESH = 'BD'
    BELGICA = 'BE'
    BURKINA_FASO = 'BF'
    BULGARIA = 'BG'
    ESTADO_DE_BAHREIN = 'BH'
    BURUNDI = 'BI'
    BENIN = 'BJ'
    BELICE = 'BL'
    BERMUDAS = 'BM'
    BRUNEI_DARUSSALAM = 'BN'
    BOLIVIA = 'BO'
    BRASIL = 'BR'
    COMMONWEALTH_DE_LAS_BAHAMAS = 'BS'
    BUTHAN = 'BT'
    BURMA = 'BU'
    ISLA_BOUVET = 'BV'
    BOTSWANA = 'BW'
    BIELORRUSIA = 'BY'
    CANADA = 'CA'
    ISLA_DE_COCOS_O_KELLING = 'CC'
    ISLAS_CANARIAS = 'CD'
    ISLA_DE_CHRISTMAS = 'CE'
    REPUBLICA_CENTRO_AFRICANA = 'CF'
    CONGO = 'CG'
    SUIZA = 'CH'
    COSTA_DE_MARFIL = 'CI'
    ISLAS_COOK = 'CK'
    CHILE = 'CL'
    CAMERUN = 'CM'
    CHINA = 'CN'
    COLOMBIA = 'CO'
    CAMPIONE_DITALIA = 'CP'
    REPUBLICA_DE_COSTA_RICA = 'CR'
    REPUBLICA_CHECA_Y_REPUBLICA_ESLOVACA = 'CS'
    CUBA = 'CU'
    REPUBLICA_DE_CABO_VERDE = 'CV'
    ISLA_DE_NAVIDAD = 'CX'
    REPUBLICA_DE_CHIPRE = 'CY'
    ALEMANIA = 'DD'
    REPUBLICA_DE_DJIBOUTI = 'DJ'
    DINAMARCA = 'DK'
    REPUBLICA_DOMINICANA = 'DM'
    COMMONWEALTH_DE_DOMINICA = 'DN'
    ARGELIA = 'DZ'
    ECUADOR = 'EC'
    EGIPTO = 'EG'
    SAHARA_DEL_OESTE = 'EH'
    ESTADO_INDEPENDIENTE_DE_SAMOA_OCCIDENTAL = 'EO'
    ESPANA = 'ES'
    ETIOPIA = 'ET'
    FINLANDIA = 'FI'
    FIJI = 'FJ'
    ISLAS_MALVINAS = 'FK'
    MICRONESIA = 'FM'
    ISLAS_FAROE = 'FO'
    FRANCIA = 'FR'
    GABON = 'GA'
    GRAN_BRETANA_REINO_UNIDO = 'GB'
    GRANADA = 'GD'
    GUYANA_FRANCESA = 'GF'
    GHANA = 'GH'
    GIBRALTAR = 'GI'
    GROENLANDIA = 'GJ'
    GAMBIA = 'GM'
    GUINEA = 'GN'
    GUADALUPE = 'GP'
    GUINEA_ECUATORIAL = 'GQ'
    GRECIA = 'GR'
    GUATEMALA = 'GT'
    GUAM = 'GU'
    GUINEA_BISSAU = 'GW'
    REPUBLICA_DE_GUYANA = 'GY'
    ISLAS_DE_GUERNESEY_JERSEY_ALDERNEY_ISLA_GREAT_SARK_HERM_LITTLE_SARK_BERCHOU_JETHOU_LIHOU_ISLAS_DEL_CANAL = 'GZ'
    HONG_KONG = 'HK'
    ISLAS_HEARD_AND_MC_DONALD = 'HM'
    REPUBLICA_DE_HONDURAS = 'HN'
    HAITI = 'HT'
    HUNGRIA = 'HU'
    INDONESIA = 'ID'
    IRLANDA = 'IE'
    ISLA_DEL_HOMBRE = 'IH'
    ISRAEL = 'IL'
    INDIA = 'IN'
    TERRITORIO_BRITANICO_EN_EL_OCEANO_INDICO = 'IO'
    ISLAS_PACIFICO = 'IP'
    IRAQ = 'IQ'
    IRAN = 'IR'
    ISLANDIA = 'IS'
    ITALIA = 'IT'
    JAMAICA = 'JM'
    REINO_HACHEMITA_DE_JORDANIA = 'JO'
    JAPON = 'JP'
    KENIA = 'KE'
    CAMPUCHEA_DEMOCRATICA = 'KH'
    KIRIBATI = 'KI'
    COMOROS = 'KM'
    SAN_KITTS = 'KN'
    REPUBLICA_DEMOCRATICA_DE_COREA = 'KP'
    REPUBLICA_DE_COREA = 'KR'
    ESTADO_DE_KUWAIT = 'KW'
    ISLAS_CAIMAN = 'KY'
    REPUBLICA_DEMOCRATICA_DE_LAOS = 'LA'
    LIBANO = 'LB'
    SANTA_LUCIA = 'LC'
    PRINCIPADO_DE_LIECHTENSTEIN = 'LI'
    REPUBLICA_SOCIALISTA_DEMOCRATICA_DE_SRI_LANKA = 'LK'
    LABUAN = 'LN'
    REPUBLICA_DE_LIBERIA = 'LR'
    LESOTHO = 'LS'
    GRAN_DUCADO_DE_LUXEMBURGO = 'LU'
    LIBIA = 'LY'
    MARRUECOS = 'MA'
    PRINCIPADO_DE_MONACO = 'MC'
    MADEIRA = 'MD'
    MADAGASCAR = 'MG'
    REPUBLICA_DE_LAS_ISLAS_MARSHALL = 'MH'
    MALI = 'ML'
    MONGOLIA = 'MN'
    MACAO = 'MO'
    ISLAS_MARIANAS_DEL_NORESTE = 'MP'
    MARTINICA = 'MQ'
    MAURITANIA = 'MR'
    MONSERRAT = 'MS'
    MALTA = 'MT'
    REPUBLICA_DE_MAURICIO = 'MU'
    REPUBLICA_DE_MALDIVAS = 'MV'
    MALAWI = 'MW'
    MALASIA_ = 'MY'
    MOZAMBIQUE = 'MZ'
    REPUBLICA_DE_NAMIBIA = 'NA'
    NUEVA_CALEDONIA = 'NC'
    NIGER = 'NE'
    ISLA_DE_NORFOLK = 'NF'
    NIGERIA = 'NG'
    NICARAGUA = 'NI'
    HOLANDA = 'NL'
    NORUEGA = 'NO'
    NEPAL = 'NP'
    REPUBLICA_DE_NAURU = 'NR'
    ZONA_NEUTRAL = 'NT'
    NIUE = 'NU'
    NEVIS = 'NV'
    NUEVA_ZELANDIA = 'NZ'
    SULTANIA_DE_OMAN = 'OM'
    REPUBLICA_DE_PANAMA = 'PA'
    PERU = 'PE'
    POLINESIA_FRANCESA = 'PF'
    PAPUA_NUEVA_GUINEA = 'PG'
    FILIPINAS = 'PH'
    PAKISTAN = 'PK'
    POLONIA = 'PL'
    ISLA_DE_SAN_PEDRO_Y_MIGUELON = 'PM'
    PITCAIRN = 'PN'
    ESTADO_LIBRE_ASOCIADO_DE_PUERTO_RICO = 'PR'
    PORTUGAL = 'PT'
    PATAU = 'PU'
    PALAU = 'PW'
    PARAGUAY = 'PY'
    ESTADO_DE_QUATAR = 'QA'
    ISLA_QESHM = 'QB'
    REUNION = 'RE'
    RUMANIA = 'RO'
    RHUANDA = 'RW'
    ARABIA_SAUDITA = 'SA'
    ISLAS_SALOMON = 'SB'
    SEYCHELLES_ISLAS = 'SC'
    SUDAN = 'SD'
    SUECIA = 'SE'
    SINGAPUR = 'SG'
    SANTA_ELENA = 'SH'
    ARCHIPIELAGO_DE_SVALBARD = 'SI'
    ISLAS_SVALBARD_AND_JAN_MAYEN = 'SJ'
    SARK = 'SK'
    SIERRA_LEONA = 'SL'
    SERENISIMA_REPUBLICA_DE_SAN_MARINO = 'SM'
    SENEGAL = 'SN'
    SOMALIA = 'SO'
    SURINAM = 'SR'
    SAO_TOME_AND_PRINCIPE = 'ST'
    PAISES_DE_LA_EX_URSS_EXCEPTO_UCRANIA_Y_BIELORUSIA = 'SU'
    EL_SALVADOR = 'SV'
    REPUBLICA_DE_SEYCHELLES = 'SW'
    SIRIA = 'SY'
    REINO_DE_SWAZILANDIA = 'SZ'
    ISLAS_TURCAS_Y_CAICOS = 'TC'
    CHAD = 'TD'
    TERRITORIOS_FRANCESES_DEL_SURESTE = 'TF'
    TOGO = 'TG'
    THAILANDIA = 'TH'
    TOKELAU = 'TK'
    REPUBLICA_DE_TUNEZ = 'TN'
    REINO_DE_TONGA = 'TO'
    TIMOR_ESTE = 'TP'
    TRIESTE = 'TR'
    TRISTAN_DA_CUNHA = 'TS'
    REPUBLICA_DE_TRINIDAD_Y_TOBAGO = 'TT'
    TURQUIA = 'TU'
    TUVALU = 'TV'
    TAIWAN = 'TW'
    TANZANIA = 'TZ'
    UCRANIA = 'UA'
    UGANDA = 'UG'
    ISLAS_MENORES_ALEJADAS_DE_LOS_ESTADOS_UNIDOS = 'UM'
    ESTADOS_UNIDOS_DE_AMERICA = 'US'
    REPUBLICA_ORIENTAL_DEL_URUGUAY = 'UY'
    EL_VATICANO = 'VA'
    SAN_VICENTE_Y_LAS_GRANADINAS = 'VC'
    VENEZUELA = 'VE'
    ISLAS_VIRGENES_BRITANICAS = 'VG'
    ISLAS_VIRGENES_DE_ESTADOS_UNIDOS_DE_AMERICA = 'VI'
    VIETNAM = 'VN'
    REPUBLICA_DE_VANUATU = 'VU'
    ISLAS_WALLIS_Y_FUNTUNA = 'WF'
    OTRO = 'XX'
    YEMEN_DEMOCRATICA = 'YD'
    REPUBLICA_DEL_YEMEN = 'YE'
    PAISES_DE_LA_EX_YUGOSLAVIA = 'YU'
    SUDAFRICA = 'ZA'
    ZONA_ESPECIAL_CANARIA = 'ZC'
    ZAMBIA = 'ZM'
    ZONA_LIBRE_OSTRAVA = 'ZO'
    ZAIRE = 'ZR'
    ZIMBAWE = 'ZW'
