from ..utils import iterate
from .. import NamespaceMismatchError


def ubicacion0(col, data):
    col.add_map('dvz', 'http://www.diverza.com/ns/addenda/diverza/1')
    col.add_schema('http://www.diverza.com/ns/addenda/diverza/1 http://www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    col.add_base('www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    pass
def datos_contacto0(col, data):
    col.add_map('dvz', 'http://www.diverza.com/ns/addenda/diverza/1')
    col.add_schema('http://www.diverza.com/ns/addenda/diverza/1 http://www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    col.add_base('www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    pass
def extra0(col, data):
    col.add_map('dvz', 'http://www.diverza.com/ns/addenda/diverza/1')
    col.add_schema('http://www.diverza.com/ns/addenda/diverza/1 http://www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    col.add_base('www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    pass
def archivo_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def informe_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def sujeto_obligado_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def aviso_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def modificatorio_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def alerta_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def persona_aviso_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def tipo_persona_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def persona_fisica_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def persona_moral_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def fideicomiso_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def representante_apoderado_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def tipo_domicilio_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def nacional_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def extranjero_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def telefono_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def dueno_beneficiario_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def tipo_persona_simple_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def persona_fisica_simple_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def persona_moral_simple_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def fideicomiso_simple_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def detalle_operaciones_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def datos_operacion_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def caracteristicas_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def datos_liquidacion_type0(col, data):
    col.add_map('ari', 'http://www.uif.shcp.gob.mx/recepcion/ari')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/ari ari.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/ari.xsd')
    pass
def archivo_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def informe_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def sujeto_obligado_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def aviso_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def modificatorio_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def alerta_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def operaciones_persona_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_aviso_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def datos_cuenta_plataforma_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def tipo_persona_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_fisica_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_moral_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def fideicomiso_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def representante_apoderado_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def documento_identificacion_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def tipo_domicilio_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def nacional_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def extranjero_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def telefono_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def dueno_beneficiario_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def tipo_persona_simple_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_fisica_simple_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_moral_simple_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def fideicomiso_simple_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def detalle_operaciones_type1(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def operaciones_compra_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def operaciones_venta_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def operaciones_intercambio_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def operaciones_transferencia_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def transferencias_enviadas_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def transferencias_recibidas_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def operaciones_fondos_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def fondos_retirados_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def fondos_depositados_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def compra_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def venta_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def intercambio_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def envio_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def recepcion_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def retiro_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def deposito_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def activo_virtual_simple_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def activo_virtual_compuesto_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_cuenta_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def tipo_persona_basico_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_fisica_basico_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def persona_moral_basico_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def pais_cuenta_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def cuenta_nacional_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def cuenta_extranjera_type0(col, data):
    col.add_map('avi', 'http://www.uif.shcp.gob.mx/recepcion/avi')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/avi avi.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/avi.xsd')
    pass
def archivo_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def informe_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def sujeto_obligado_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def aviso_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def modificatorio_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def alerta_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def persona_aviso_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def tipo_persona_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def persona_fisica_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def persona_moral_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def fideicomiso_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def representante_apoderado_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def tipo_domicilio_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def nacional_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def extranjero_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def telefono_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def dueno_beneficiario_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def tipo_persona_simple_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def persona_fisica_simple_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def persona_moral_simple_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def fideicomiso_simple_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def detalle_operaciones_type2(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def datos_operacion_type1(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def tipo_bien_type0(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def datos_vehiculo_terrestre_type0(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def datos_inmueble_type0(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def datos_parte_blindada_type0(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def datos_liquidacion_type1(col, data):
    col.add_map('bli', 'http://www.uif.shcp.gob.mx/recepcion/bli')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/bli bli.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/bli.xsd')
    pass
def archivo_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def informe_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def sujeto_obligado_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def aviso_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def modificatorio_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def alerta_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def persona_aviso_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def tipo_persona_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def persona_fisica_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def persona_moral_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def fideicomiso_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def representante_apoderado_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def tipo_domicilio_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def nacional_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def extranjero_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def telefono_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def dueno_beneficiario_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def tipo_persona_simple_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def persona_fisica_simple_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def persona_moral_simple_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def fideicomiso_simple_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def detalle_operaciones_type3(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def datos_operacion_type2(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def datos_cheque_type0(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def datos_liquidacion_type2(col, data):
    col.add_map('chv', 'http://www.uif.shcp.gob.mx/recepcion/chv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/chv chv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/chv.xsd')
    pass
def archivo_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def informe_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def sujeto_obligado_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def aviso_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def modificatorio_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def alerta_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def detalle_operaciones_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_operacion_type3(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def desarrollos_inmobiliarios_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_desarrollo_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def caracteristicas_desarrollo_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def aportaciones_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def tipo_aportacion_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def recursos_propios_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_aportacion_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def aportacion_numerario_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def aportacion_especie_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def socios_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def detalle_socios_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_socio_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def tipo_persona_socio_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def persona_fisica_socio_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def persona_moral_socio_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def representante_apoderado_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def tipo_domicilio_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def nacional_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def extranjero_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def telefono_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def socios_aportacion_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_aportacion_socios_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def terceros_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def detalle_terceros_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_tercero_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def tipo_persona_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def persona_fisica_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def persona_moral_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def fideicomiso_type4(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def terceros_aportacion_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_aportacion_terceros_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def aportacion_numerario_terceros_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def prestamo_financiero_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_prestamo_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def prestamo_no_financiero_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def datos_prestamo_no_fin_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def detalle_acreedores_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def financiamiento_bursatil_type0(col, data):
    col.add_map('din', 'http://www.uif.shcp.gob.mx/recepcion/din')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/din din.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/din.xsd')
    pass
def archivo_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def informe_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def sujeto_obligado_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def aviso_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def modificatorio_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def alerta_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def persona_aviso_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def tipo_persona_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def persona_fisica_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def persona_moral_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def fideicomiso_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def representante_apoderado_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def tipo_domicilio_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def nacional_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def extranjero_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def telefono_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def dueno_beneficiario_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def tipo_persona_simple_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def persona_fisica_simple_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def persona_moral_simple_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def fideicomiso_simple_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def detalle_operaciones_type5(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def datos_operacion_type4(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def datos_donativo_type0(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def tipo_donativo_type0(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def liquidacion_numerario_type0(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def liquidacion_especie_type0(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def datos_bien_donado_type0(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def datos_inmueble_type1(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def datos_otro_type0(col, data):
    col.add_map('don', 'http://www.uif.shcp.gob.mx/recepcion/don')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/don don.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/don.xsd')
    pass
def tipo_persona_1_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def archivo_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def informe_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def sujeto_obligado_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def aviso_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def modificatorio_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def alerta_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_aviso_type5(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def detalle_operaciones_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_operacion_type5(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def tipo_actividad_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def otorgamiento_poder_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_poderdante_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_apoderado_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def tipo_persona_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_fisica_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_moral_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def fideicomiso_type6(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def tipo_persona_simple_type5(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_moral_simple_type5(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_fisica_simple_type5(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def constitucion_personas_morales_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_accionista_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def capital_social_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def modificacion_patrimonial_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_moral_modifica_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_modificacion_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_accionista_1_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def fusion_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_fusionadas_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_fusionada_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_fusionante_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def fusionante_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def escision_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_escindente_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_escindidas_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def escindida_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def compra_venta_acciones_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_moral_acciones_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_vendedor_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_comprador_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_liquidacion_type3(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def const_mod_fideicomiso_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_fideicomitente_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_tipo_patrimonio_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def patrimonio_monetaria_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def patrimonio_inmueble_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def patrimonio_otro_bien_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_fideicomisario_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def cesion_derechos_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_cedente_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_cesionario_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_cesion_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def contrato_mutuo_cred_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_acreedor_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_deudor_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_garantia_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_bien_mutuo_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_inmueble_type2(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_otro_type1(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def tipo_persona_garante_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_fisica_garante_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def persona_moral_garante_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_liquidacion_simple_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def avaluo_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_propietario_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def datos_comite_tecnico_type0(col, data):
    col.add_map('fep', 'http://www.uif.shcp.gob.mx/recepcion/fep')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fep fep.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fep.xsd')
    pass
def administrativo1_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def persona_moral_simple_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def persona_fisica_simple_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def fideicomiso_simple_type5(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def representante_apoderado_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def informe_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def archivo_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def operaciones_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def operacion_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def modificatorio_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def aviso_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def actividad_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def derechos_inmueble_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def otorgamiento_poder_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def constitucion_pm_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def modificacion_patrimonial_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def contrato_mutuo_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def avaluo_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_inmuebles_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def inmueble_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def personas_acto_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tipo_persona_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def persona_fisica_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def persona_moral_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def fideicomiso_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_persona_acto_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tipo_domicilio_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def nacional_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def extranjero_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def telefono_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def administrativo_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def jurisdiccional_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tipo_autoridad_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def domicilio_oficina_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def autoridad_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tipo_persona_simple_type6(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_poderdante_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_apoderado_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tipo_autoridad1_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def autoridad1_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def persona_moral_const_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_accionista_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def capital_social_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_accionista1_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def persona_moral_modifica_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_modificacion_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_acreedor_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_deudor_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def garante_fisica_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def garante_moral_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def garante_fideicomiso_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tipo_garante_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def garantia_inmueble_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def garantia_otro_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def bien_garantia_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_garantia_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_liquidacion_type4(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def datos_propietario_type1(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def propietario_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def tribunal_type0(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def alerta_type7(col, data):
    col.add_map('fes', 'http://www.uif.shcp.gob.mx/recepcion/fes')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/fes fes.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/fes.xsd')
    pass
def archivo_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def informe_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def sujeto_obligado_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def aviso_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def modificatorio_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def alerta_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def persona_aviso_type6(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def tipo_persona_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def persona_fisica_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def persona_moral_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def fideicomiso_type8(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def representante_apoderado_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def tipo_domicilio_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def nacional_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def extranjero_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def telefono_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def dueno_beneficiario_type5(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def tipo_persona_simple_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def persona_fisica_simple_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def persona_moral_simple_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def fideicomiso_simple_type6(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def detalle_operaciones_type7(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def datos_operacion_type6(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def datos_contraparte_type0(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def caracteristicas_inmueble_type0(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def contrato_instrumento_type0(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def datos_instrumento_type0(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def datos_contrato_type0(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def datos_liquidacion_type5(col, data):
    col.add_map('inm', 'http://www.uif.shcp.gob.mx/recepcion/inm')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/inm inm.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/inm.xsd')
    pass
def archivo_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def informe_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def sujeto_obligado_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def aviso_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def modificatorio_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def alerta_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def persona_aviso_type7(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def tipo_persona_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def persona_fisica_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def persona_moral_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def fideicomiso_type9(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def representante_apoderado_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def tipo_domicilio_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def nacional_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def extranjero_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def telefono_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def dueno_beneficiario_type6(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def tipo_persona_simple_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def persona_fisica_simple_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def persona_moral_simple_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def fideicomiso_simple_type7(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def detalle_operaciones_type8(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_operacion_type7(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def tipo_sucursal_type0(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_sucursal_propia_type0(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_sucursal_operador_type0(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_liquidacion_type6(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def liquidacion_numerario_type1(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def liquidacion_especie_type1(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_bien_liquidacion_type0(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_inmueble_type3(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def datos_otro_type2(col, data):
    col.add_map('jys', 'http://www.uif.shcp.gob.mx/recepcion/jys')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/jys jys.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/jys.xsd')
    pass
def archivo_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def informe_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def sujeto_obligado_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def aviso_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def modificatorio_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def alerta_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def persona_aviso_type8(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def tipo_persona_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def persona_fisica_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def persona_moral_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def fideicomiso_typea(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def representante_apoderado_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def tipo_domicilio_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def nacional_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def extranjero_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def telefono_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def dueno_beneficiario_type7(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def tipo_persona_simple_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def persona_fisica_simple_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def persona_moral_simple_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def fideicomiso_simple_type8(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def detalle_operaciones_type9(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def datos_operacion_type8(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def datos_bien_type0(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def datos_liquidacion_type7(col, data):
    col.add_map('mjr', 'http://www.uif.shcp.gob.mx/recepcion/mjr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mjr mjr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mjr.xsd')
    pass
def archivo_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def informe_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def sujeto_obligado_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def aviso_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def modificatorio_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def alerta_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_aviso_type9(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def tipo_persona_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_fisica_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_moral_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def fideicomiso_typeb(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def representante_apoderado_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def tipo_domicilio_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def nacional_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def extranjero_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def telefono_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def dueno_beneficiario_type8(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def tipo_persona_simple_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_fisica_simple_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_moral_simple_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def fideicomiso_simple_type9(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def detalle_operaciones_typea(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_operacion_type9(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_garantia_type2(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_bien_mutuo_type1(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_inmueble_type4(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_otro_type3(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def tipo_garante_type1(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_fisica_garante_type1(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def persona_moral_garante_type1(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def fideicomiso_garante_type0(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_liquidacion_type8(col, data):
    col.add_map('mpc', 'http://www.uif.shcp.gob.mx/recepcion/mpc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/mpc mpc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/mpc.xsd')
    pass
def datos_liquidacion_type9(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def detalle_operaciones_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def datos_operacion_typea(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def datos_objeto_type0(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def archivo_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def informe_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def sujeto_obligado_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def aviso_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def modificatorio_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def alerta_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def persona_aviso_typea(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def tipo_persona_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def persona_fisica_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def persona_moral_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def fideicomiso_typec(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def representante_apoderado_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def tipo_domicilio_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def nacional_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def extranjero_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def telefono_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def dueno_beneficiario_type9(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def tipo_persona_simple_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def persona_fisica_simple_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def persona_moral_simple_typeb(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def fideicomiso_simple_typea(col, data):
    col.add_map('oba', 'http://www.uif.shcp.gob.mx/recepcion/oba')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/oba oba.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/oba.xsd')
    pass
def archivo_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def informe_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def sujeto_obligado_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def ocupacion_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def aviso_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def modificatorio_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def alerta_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_aviso_typeb(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_persona_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_fisica_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_moral_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def fideicomiso_typed(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def representante_apoderado_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_domicilio_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def nacional_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def extranjero_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def telefono_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def dueno_beneficiario_typea(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_persona_simple_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_fisica_simple_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_moral_simple_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def fideicomiso_simple_typeb(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def detalle_operaciones_typec(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_operacion_typeb(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_actividad_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def compra_venta_inmuebles_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_contraparte_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_persona_t1_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_fisica_t1_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_moral_t1_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def caracteristicas_inmueble_1_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def contrato_instrumento_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_instrumento_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def contrato_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def cesion_derechos_inmuebles_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def caracteristicas_inmueble_2_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def administracion_recursos_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_activo_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def activo_banco_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def activo_inmobiliario_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def activo_outsourcing_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def area_servicio_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def activo_administrado_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def activo_otros_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def constitucion_sociedades_mercantiles_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_accionista_t1_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def capital_social_type2(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def organizacion_aportaciones_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_aportacion_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_persona_t2_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_fisica_t2_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def persona_moral_t2_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_tipo_aportacion_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def aportacion_monetaria_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def aportacion_inmueble_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def aportacion_otro_bien_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def fusion_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_fusionadas_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_fusionada_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_fusionante_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def fusionante_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_accionista_t2_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def escision_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_escindente_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_accionista_t3_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_escindidas_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def escindida_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def admon_personas_morales_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def tipo_persona_moral_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def constitucion_fideicomiso_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_fideicomitente_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_tipo_patrimonio_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def patrimonio_monetario_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def patrimonio_inmueble_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def patrimonio_otro_bien_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_fideicomisario_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_comite_tecnico_type1(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def compra_venta_entidades_mercantiles_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_sociedad_mercantil_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def datos_operacion_financiera_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def activo_virtual_type0(col, data):
    col.add_map('spr', 'http://www.uif.shcp.gob.mx/recepcion/spr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/spr spr2.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/spr2.xsd')
    pass
def archivo_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def informe_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def sujeto_obligado_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def aviso_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def modificatorio_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def alerta_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_aviso_typec(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_persona_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_fisica_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_moral_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def fideicomiso_typee(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def representante_apoderado_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_domicilio_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def nacional_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def extranjero_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def telefono_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def dueno_beneficiario_typeb(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_persona_simple_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_fisica_simple_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_moral_simple_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def fideicomiso_simple_typec(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def detalle_operaciones_typed(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def datos_operacion_typec(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_bien_type1(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def datos_efectivo_instrumentos_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def datos_valores_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def recepcion_type1(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def custodia_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_custodia_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def datos_sucursal_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def datos_no_sucursal_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def entrega_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_entrega_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def nacional_entrega_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def extranjero_entrega_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def destinatario_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def tipo_persona_destinatario_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_fisica_destinatario_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def persona_moral_destinatario_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def fideicomiso_destinatario_type0(col, data):
    col.add_map('tcv', 'http://www.uif.shcp.gob.mx/recepcion/tcv')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tcv tcv.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tcv.xsd')
    pass
def archivo_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def informe_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def sujeto_obligado_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def aviso_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def modificatorio_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def alerta_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def persona_aviso_typed(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def tipo_persona_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def persona_fisica_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def persona_moral_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def fideicomiso_typef(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def representante_apoderado_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def tipo_domicilio_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def nacional_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def extranjero_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def telefono_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def dueno_beneficiario_typec(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def tipo_persona_simple_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def persona_fisica_simple_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def persona_moral_simple_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def fideicomiso_simple_typed(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def detalle_operaciones_typee(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def datos_operacion_typed(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def datos_liquidacion_typea(col, data):
    col.add_map('tdr', 'http://www.uif.shcp.gob.mx/recepcion/tdr')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tdr tdr.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tdr.xsd')
    pass
def archivo_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def informe_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def sujeto_obligado_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def aviso_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def modificatorio_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def alerta_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def persona_aviso_typee(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def tipo_persona_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def persona_fisica_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def persona_moral_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def fideicomiso_type10(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def representante_apoderado_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def tipo_domicilio_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def nacional_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def extranjero_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def telefono_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def dueno_beneficiario_typed(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def tipo_persona_simple_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def persona_fisica_simple_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def persona_moral_simple_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def fideicomiso_simple_typee(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def detalle_operaciones_typef(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def datos_operacion_typee(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def datos_liquidacion_typeb(col, data):
    col.add_map('tpp', 'http://www.uif.shcp.gob.mx/recepcion/tpp')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tpp tpp.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tpp.xsd')
    pass
def detalle_operaciones_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def datos_operacion_typef(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def archivo_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def informe_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def sujeto_obligado_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def aviso_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def modificatorio_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def alerta_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def persona_aviso_typef(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def tipo_persona_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def persona_fisica_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def persona_moral_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def fideicomiso_type11(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def representante_apoderado_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def tipo_domicilio_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def nacional_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def extranjero_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def telefono_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def dueno_beneficiario_typee(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def tipo_persona_simple_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def persona_fisica_simple_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def persona_moral_simple_type10(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def fideicomiso_simple_typef(col, data):
    col.add_map('tsc', 'http://www.uif.shcp.gob.mx/recepcion/tsc')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/tsc tsc.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/tsc.xsd')
    pass
def archivo_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def informe_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def sujeto_obligado_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def aviso_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def modificatorio_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def alerta_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def persona_aviso_type10(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def tipo_persona_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def persona_fisica_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def persona_moral_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def fideicomiso_type12(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def representante_apoderado_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def tipo_domicilio_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def nacional_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def extranjero_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def telefono_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def dueno_beneficiario_typef(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def tipo_persona_simple_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def persona_fisica_simple_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def persona_moral_simple_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def fideicomiso_simple_type10(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def detalle_operaciones_type11(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def datos_operacion_type10(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def tipo_vehiculo_type0(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def datos_vehiculo_terrestre_type1(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def datos_vehiculo_maritimo_type0(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def datos_vehiculo_aereo_type0(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def datos_liquidacion_typec(col, data):
    col.add_map('veh', 'http://www.uif.shcp.gob.mx/recepcion/veh')
    col.add_schema('http://www.uif.shcp.gob.mx/recepcion/veh veh.xsd')
    col.add_base('www.pld.hacienda.gob.mx/work/models/PLD/documentos/xsd/veh.xsd')
    pass
def t_ubicacion0(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    pass
def t_ubicacion_fiscal0(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    pass
def t_informacion_aduanera0(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    pass
def t_ubicacion_fiscal1(col, data):
    col.add_map('terceros', 'http://www.sat.gob.mx/terceros')
    col.add_schema('http://www.sat.gob.mx/terceros http://www.sat.gob.mx/sitio_internet/cfd/terceros/terceros11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/terceros/terceros11.xsd')
    pass
def t_informacion_aduanera1(col, data):
    col.add_map('terceros', 'http://www.sat.gob.mx/terceros')
    col.add_schema('http://www.sat.gob.mx/terceros http://www.sat.gob.mx/sitio_internet/cfd/terceros/terceros11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/terceros/terceros11.xsd')
    pass
def t_informacion_aduanera2(col, data):
    col.add_map('vehiculousado', 'http://www.sat.gob.mx/vehiculousado')
    col.add_schema('http://www.sat.gob.mx/vehiculousado http://www.sat.gob.mx/sitio_internet/cfd/vehiculousado/vehiculousado.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/vehiculousado/vehiculousado.xsd')
    pass
def t_informacion_aduanera3(col, data):
    col.add_map('ventavehiculos', 'http://www.sat.gob.mx/ventavehiculos')
    col.add_schema('http://www.sat.gob.mx/ventavehiculos http://www.sat.gob.mx/sitio_internet/cfd/ventavehiculos/ventavehiculos11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ventavehiculos/ventavehiculos11.xsd')
    pass
def signature_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def signature_value_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def signed_info_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def canonicalization_method_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def signature_method_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def reference_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def transforms_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def transform_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def digest_method_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def key_info_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def key_value_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def retrieval_method_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def x509data_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def x509issuer_serial_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def pgpdata_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def spkid_ata_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def object_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def manifest_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def signature_properties_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def signature_property_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def dsakey_value_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def rsakey_value_type0(col, data):
    col.add_map('ds', 'http://www.w3.org/2000/09/xmldsig#')
    col.add_schema('http://www.w3.org/2000/09/xmldsig# www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    col.add_base('www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd')
    pass
def cancelacion0(col, data):
    col.add_map(None, 'http://cancelacfd.sat.gob.mx')
    col.add_schema('http://cancelacfd.sat.gob.mx CancelaCfdi.xsd')
    col.add_base('CancelaCfdi.xsd')
def cancelacion1(col, data):
    col.add_map(None, 'http://www.sat.gob.mx/esquemas/retencionpago/1')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1 CancelaRetencion.xsd')
    col.add_base('CancelaRetencion.xsd')
def solicitud_aceptacion_rechazo0(col, data):
    col.add_map(None, 'http://cancelacfd.sat.gob.mx')
    col.add_schema('http://cancelacfd.sat.gob.mx SolicitudAceptacionRechazo.xsd')
    col.add_base('SolicitudAceptacionRechazo.xsd')
def diverza0(col, data):
    col.add_map('dvz', 'http://www.diverza.com/ns/addenda/diverza/1')
    col.add_schema('http://www.diverza.com/ns/addenda/diverza/1 http://www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
    col.add_base('www.diverza.com/schema/xsd/Addenda_Diverza_v1.1.xsd')
def auxiliar_ctas0(col, data):
    col.add_map('AuxiliarCtas', 'www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas')
    col.add_schema('www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas/AuxiliarCtas_1_1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas/AuxiliarCtas_1_1.xsd')
def rep_aux_fol0(col, data):
    col.add_map('RepAux', 'www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios')
    col.add_schema('www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios/AuxiliarFolios_1_2.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios/AuxiliarFolios_1_2.xsd')
def balanza0(col, data):
    col.add_map('BCE', 'www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion')
    col.add_schema('www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion/BalanzaComprobacion_1_1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion/BalanzaComprobacion_1_1.xsd')
def catalogo0(col, data):
    col.add_map('catalogocuentas', 'www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas')
    col.add_schema('www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas/CatalogoCuentas_1_1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas/CatalogoCuentas_1_1.xsd')
def polizas0(col, data):
    col.add_map('PLZ', 'www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo')
    col.add_schema('www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo/PolizasPeriodo_1_1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo/PolizasPeriodo_1_1.xsd')
def auxiliar_ctas1(col, data):
    col.add_map('AuxiliarCtas', 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas')
    col.add_schema('http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas/AuxiliarCtas_1_3.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas/AuxiliarCtas_1_3.xsd')
def rep_aux_fol1(col, data):
    col.add_map('RepAux', 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios')
    col.add_schema('http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios/AuxiliarFolios_1_3.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios/AuxiliarFolios_1_3.xsd')
def balanza1(col, data):
    col.add_map('BCE', 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion')
    col.add_schema('http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion/BalanzaComprobacion_1_3.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion/BalanzaComprobacion_1_3.xsd')
def catalogo1(col, data):
    col.add_map('catalogocuentas', 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas')
    col.add_schema('http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas/CatalogoCuentas_1_3.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas/CatalogoCuentas_1_3.xsd')
def polizas1(col, data):
    col.add_map('PLZ', 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo')
    col.add_schema('http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo/PolizasPeriodo_1_3.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo/PolizasPeriodo_1_3.xsd')
def sello_digital_cont_elec0(col, data):
    col.add_map('sellodigital', 'www.sat.gob.mx/esquemas/ContabilidadE/1_1/SelloDigitalContElec')
    col.add_schema('www.sat.gob.mx/esquemas/ContabilidadE/1_1/SelloDigitalContElec http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/SelloDigitalContElec/SelloDigitalContElec.xsd')
    col.add_base('www.sat.gob.mx/esquemas/ContabilidadE/1_3/SelloDigitalContElec/SelloDigitalContElec.xsd')
def servicios_plataformas_tecnologicas0(col, data):
    col.add_map('plataformasTecnologicas', 'http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10 http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10/ServiciosPlataformasTecnologicas10.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10/ServiciosPlataformasTecnologicas10.xsd')
def arrendamientoenfideicomiso0(col, data):
    col.add_map('arrendamientoenfideicomiso', 'http://www.sat.gob.mx/esquemas/retencionpago/1/arrendamientoenfideicomiso')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/arrendamientoenfideicomiso http://www.sat.gob.mx/esquemas/retencionpago/1/arrendamientoenfideicomiso/arrendamientoenfideicomiso.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/arrendamientoenfideicomiso/arrendamientoenfideicomiso.xsd')
def dividendos0(col, data):
    col.add_map('dividendos', 'http://www.sat.gob.mx/esquemas/retencionpago/1/dividendos')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/dividendos http://www.sat.gob.mx/esquemas/retencionpago/1/dividendos/dividendos.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/dividendos/dividendos.xsd')
def enajenacionde_acciones0(col, data):
    col.add_map('enajenaciondeacciones', 'http://www.sat.gob.mx/esquemas/retencionpago/1/enajenaciondeacciones')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/enajenaciondeacciones http://www.sat.gob.mx/esquemas/retencionpago/1/enajenaciondeacciones/enajenaciondeacciones.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/enajenaciondeacciones/enajenaciondeacciones.xsd')
def fideicomisonoempresarial0(col, data):
    col.add_map('fideicomisonoempresarial', 'http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial/fideicomisonoempresarial.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial/fideicomisonoempresarial.xsd')
def intereses0(col, data):
    col.add_map('intereses', 'http://www.sat.gob.mx/esquemas/retencionpago/1/intereses')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/intereses http://www.sat.gob.mx/esquemas/retencionpago/1/intereses/intereses.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/intereses/intereses.xsd')
def intereseshipotecarios0(col, data):
    col.add_map('intereseshipotecarios', 'http://www.sat.gob.mx/esquemas/retencionpago/1/intereseshipotecarios')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/intereseshipotecarios http://www.sat.gob.mx/esquemas/retencionpago/1/intereseshipotecarios/intereseshipotecarios.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/intereseshipotecarios/intereseshipotecarios.xsd')
def operacionesconderivados0(col, data):
    col.add_map('operacionesconderivados', 'http://www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados http://www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados/operacionesconderivados.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados/operacionesconderivados.xsd')
def pagosaextranjeros0(col, data):
    col.add_map('pagosaextranjeros', 'http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros/pagosaextranjeros.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros/pagosaextranjeros.xsd')
def planesderetiro0(col, data):
    col.add_map('planesderetiro11', 'http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro11')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro11 http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro11/planesderetiro11.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro11/planesderetiro11.xsd')
def planesderetiro1(col, data):
    col.add_map('planesderetiro', 'http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro/planesderetiro.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro/planesderetiro.xsd')
def premios0(col, data):
    col.add_map('premios', 'http://www.sat.gob.mx/esquemas/retencionpago/1/premios')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/premios http://www.sat.gob.mx/esquemas/retencionpago/1/premios/premios.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/premios/premios.xsd')
def retenciones0(col, data):
    col.add_map('retenciones', 'http://www.sat.gob.mx/esquemas/retencionpago/1')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1 http://www.sat.gob.mx/esquemas/retencionpago/1/retencionpagov1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/retencionpagov1.xsd')
    el = data.get('Complemento')
    if el is not None:
        complemento0(col, el)
    el = data.get('Addenda')
    if el is not None:
        addenda0(col, el)
def complemento0(col, data):
    col.add_map('retenciones', 'http://www.sat.gob.mx/esquemas/retencionpago/1')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1 http://www.sat.gob.mx/esquemas/retencionpago/1/retencionpagov1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/retencionpagov1.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def addenda0(col, data):
    col.add_map('retenciones', 'http://www.sat.gob.mx/esquemas/retencionpago/1')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1 http://www.sat.gob.mx/esquemas/retencionpago/1/retencionpagov1.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/retencionpagov1.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def sector_financiero0(col, data):
    col.add_map('sectorfinanciero', 'http://www.sat.gob.mx/esquemas/retencionpago/1/sectorfinanciero')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/1/sectorfinanciero http://www.sat.gob.mx/esquemas/retencionpago/1/sectorfinanciero/sectorfinanciero.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/1/sectorfinanciero/sectorfinanciero.xsd')
def retenciones1(col, data):
    col.add_map('retenciones', 'http://www.sat.gob.mx/esquemas/retencionpago/2')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/2 http://www.sat.gob.mx/esquemas/retencionpago/2/retencionpagov2.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/2/retencionpagov2.xsd')
    el = data.get('Complemento')
    if el is not None:
        complemento1(col, el)
    el = data.get('Addenda')
    if el is not None:
        addenda1(col, el)
def complemento1(col, data):
    col.add_map('retenciones', 'http://www.sat.gob.mx/esquemas/retencionpago/2')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/2 http://www.sat.gob.mx/esquemas/retencionpago/2/retencionpagov2.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/2/retencionpagov2.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def addenda1(col, data):
    col.add_map('retenciones', 'http://www.sat.gob.mx/esquemas/retencionpago/2')
    col.add_schema('http://www.sat.gob.mx/esquemas/retencionpago/2 http://www.sat.gob.mx/esquemas/retencionpago/2/retencionpagov2.xsd')
    col.add_base('www.sat.gob.mx/esquemas/retencionpago/2/retencionpagov2.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def comprobante0(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    el = data['Conceptos']
    for r in iterate(el):
        concepto0(col, r)
    el = data.get('Complemento')
    if el is not None:
        complemento2(col, el)
    el = data.get('Addenda')
    if el is not None:
        addenda2(col, el)
def concepto0(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    el = data.get('ComplementoConcepto')
    if el is not None:
        complemento_concepto0(col, el)
def complemento_concepto0(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def complemento2(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def addenda2(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def comprobante1(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    el = data['Conceptos']
    for r in iterate(el):
        concepto1(col, r)
    el = data.get('Complemento')
    if el is not None:
        complemento3(col, el)
    el = data.get('Addenda')
    if el is not None:
        addenda3(col, el)
def concepto1(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    el = data.get('ComplementoConcepto')
    if el is not None:
        complemento_concepto1(col, el)
def complemento_concepto1(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def complemento3(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def addenda3(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/3')
    col.add_schema('http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def comprobante2(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/4')
    col.add_schema('http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    el = data['Conceptos']
    for r in iterate(el):
        concepto2(col, r)
    el = data.get('Complemento')
    if el is not None:
        complemento4(col, el)
    el = data.get('Addenda')
    if el is not None:
        addenda4(col, el)
def concepto2(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/4')
    col.add_schema('http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    el = data.get('ComplementoConcepto')
    if el is not None:
        complemento_concepto2(col, el)
def complemento_concepto2(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/4')
    col.add_schema('http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def complemento4(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/4')
    col.add_schema('http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def addenda4(col, data):
    col.add_map('cfdi', 'http://www.sat.gob.mx/cfd/4')
    col.add_schema('http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
    for v in iterate(data):
        cfdi_schemas[v.tag](col, v)
def carta_porte0(col, data):
    col.add_map('cartaporte', 'http://www.sat.gob.mx/CartaPorte')
    col.add_schema('http://www.sat.gob.mx/CartaPorte http://www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte.xsd')
def carta_porte1(col, data):
    col.add_map('cartaporte20', 'http://www.sat.gob.mx/CartaPorte20')
    col.add_schema('http://www.sat.gob.mx/CartaPorte20 http://www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte20.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte20.xsd')
def comercio_exterior0(col, data):
    col.add_map('cce11', 'http://www.sat.gob.mx/ComercioExterior11')
    col.add_schema('http://www.sat.gob.mx/ComercioExterior11 http://www.sat.gob.mx/sitio_internet/cfd/ComercioExterior11/ComercioExterior11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ComercioExterior11/ComercioExterior11.xsd')
def comercio_exterior1(col, data):
    col.add_map('cce', 'http://www.sat.gob.mx/ComercioExterior')
    col.add_schema('http://www.sat.gob.mx/ComercioExterior http://www.sat.gob.mx/sitio_internet/cfd/ComercioExterior/ComercioExterior10.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ComercioExterior/ComercioExterior10.xsd')
def estado_de_cuenta_combustible0(col, data):
    col.add_map('ecc11', 'http://www.sat.gob.mx/EstadoDeCuentaCombustible')
    col.add_schema('http://www.sat.gob.mx/EstadoDeCuentaCombustible http://www.sat.gob.mx/sitio_internet/cfd/EstadoDeCuentaCombustible/ecc11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/EstadoDeCuentaCombustible/ecc11.xsd')
def estado_de_cuenta_combustible1(col, data):
    col.add_map('ecc12', 'http://www.sat.gob.mx/EstadoDeCuentaCombustible12')
    col.add_schema('http://www.sat.gob.mx/EstadoDeCuentaCombustible12 http://www.sat.gob.mx/sitio_internet/cfd/EstadoDeCuentaCombustible/ecc12.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/EstadoDeCuentaCombustible/ecc12.xsd')
def gastos_hidrocarburos0(col, data):
    col.add_map('gceh', 'http://www.sat.gob.mx/GastosHidrocarburos10')
    col.add_schema('http://www.sat.gob.mx/GastosHidrocarburos10 http://www.sat.gob.mx/sitio_internet/cfd/GastosHidrocarburos10/GastosHidrocarburos10.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/GastosHidrocarburos10/GastosHidrocarburos10.xsd')
def ingresos_hidrocarburos0(col, data):
    col.add_map('ieeh', 'http://www.sat.gob.mx/IngresosHidrocarburos10')
    col.add_schema('http://www.sat.gob.mx/IngresosHidrocarburos10 http://www.sat.gob.mx/sitio_internet/cfd/IngresosHidrocarburos10/IngresosHidrocarburos.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/IngresosHidrocarburos10/IngresosHidrocarburos.xsd')
def pagos0(col, data):
    col.add_map('pago10', 'http://www.sat.gob.mx/Pagos')
    col.add_schema('http://www.sat.gob.mx/Pagos http://www.sat.gob.mx/sitio_internet/cfd/Pagos/Pagos10.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/Pagos/Pagos10.xsd')
def pagos1(col, data):
    col.add_map('pago20', 'http://www.sat.gob.mx/Pagos20')
    col.add_schema('http://www.sat.gob.mx/Pagos20 http://www.sat.gob.mx/sitio_internet/cfd/Pagos/Pagos20.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/Pagos/Pagos20.xsd')
def timbre_fiscal_digital0(col, data):
    col.add_map('tfd', 'http://www.sat.gob.mx/TimbreFiscalDigital')
    col.add_schema('http://www.sat.gob.mx/TimbreFiscalDigital http://www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/TimbreFiscalDigital.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/TimbreFiscalDigital.xsd')
def timbre_fiscal_digital1(col, data):
    col.add_map('tfd', 'http://www.sat.gob.mx/TimbreFiscalDigital')
    col.add_schema('http://www.sat.gob.mx/TimbreFiscalDigital http://www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/TimbreFiscalDigitalv11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/TimbreFiscalDigitalv11.xsd')
def turista_pasajero_extranjero0(col, data):
    col.add_map('tpe', 'http://www.sat.gob.mx/TuristaPasajeroExtranjero')
    col.add_schema('http://www.sat.gob.mx/TuristaPasajeroExtranjero http://www.sat.gob.mx/sitio_internet/cfd/TuristaPasajeroExtranjero/TuristaPasajeroExtranjero.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/TuristaPasajeroExtranjero/TuristaPasajeroExtranjero.xsd')
def acreditamiento_ieps0(col, data):
    col.add_map('aieps', 'http://www.sat.gob.mx/acreditamiento')
    col.add_schema('http://www.sat.gob.mx/acreditamiento http://www.sat.gob.mx/sitio_internet/cfd/acreditamiento/AcreditamientoIEPS10.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/acreditamiento/AcreditamientoIEPS10.xsd')
def aerolineas0(col, data):
    col.add_map('aerolineas', 'http://www.sat.gob.mx/aerolineas')
    col.add_schema('http://www.sat.gob.mx/aerolineas http://www.sat.gob.mx/sitio_internet/cfd/aerolineas/aerolineas.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/aerolineas/aerolineas.xsd')
def obrasarteantiguedades0(col, data):
    col.add_map('obrasarte', 'http://www.sat.gob.mx/arteantiguedades')
    col.add_schema('http://www.sat.gob.mx/arteantiguedades http://www.sat.gob.mx/sitio_internet/cfd/arteantiguedades/obrasarteantiguedades.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/arteantiguedades/obrasarteantiguedades.xsd')
def certificadodedestruccion0(col, data):
    col.add_map('destruccion', 'http://www.sat.gob.mx/certificadodestruccion')
    col.add_schema('http://www.sat.gob.mx/certificadodestruccion http://www.sat.gob.mx/sitio_internet/cfd/certificadodestruccion/certificadodedestruccion.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/certificadodestruccion/certificadodedestruccion.xsd')
def cfdiregistro_fiscal0(col, data):
    col.add_map('registrofiscal', 'http://www.sat.gob.mx/registrofiscal')
    col.add_schema('http://www.sat.gob.mx/registrofiscal http://www.sat.gob.mx/sitio_internet/cfd/cfdiregistrofiscal/cfdiregistrofiscal.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/cfdiregistrofiscal/cfdiregistrofiscal.xsd')
def consumo_de_combustibles0(col, data):
    col.add_map('consumodecombustibles11', 'http://www.sat.gob.mx/ConsumoDeCombustibles11')
    col.add_schema('http://www.sat.gob.mx/ConsumoDeCombustibles11 http://www.sat.gob.mx/sitio_internet/cfd/consumodecombustibles/consumodeCombustibles11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/consumodecombustibles/consumodeCombustibles11.xsd')
def consumo_de_combustibles1(col, data):
    col.add_map('consumodecombustibles', 'http://www.sat.gob.mx/consumodecombustibles')
    col.add_schema('http://www.sat.gob.mx/consumodecombustibles http://www.sat.gob.mx/sitio_internet/cfd/consumodecombustibles/consumodecombustibles.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/consumodecombustibles/consumodecombustibles.xsd')
def detallista0(col, data):
    col.add_map('detallista', 'http://www.sat.gob.mx/detallista')
    col.add_schema('http://www.sat.gob.mx/detallista http://www.sat.gob.mx/sitio_internet/cfd/detallista/detallista.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/detallista/detallista.xsd')
def divisas0(col, data):
    col.add_map('divisas', 'http://www.sat.gob.mx/divisas')
    col.add_schema('http://www.sat.gob.mx/divisas http://www.sat.gob.mx/sitio_internet/cfd/divisas/divisas.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/divisas/divisas.xsd')
def donatarias0(col, data):
    col.add_map('donat', 'http://www.sat.gob.mx/donat')
    col.add_schema('http://www.sat.gob.mx/donat http://www.sat.gob.mx/sitio_internet/cfd/donat/donat11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/donat/donat11.xsd')
def estado_de_cuenta_bancario0(col, data):
    col.add_map('ecb', 'http://www.sat.gob.mx/ecb')
    col.add_schema('http://www.sat.gob.mx/ecb http://www.sat.gob.mx/sitio_internet/cfd/ecb/ecb.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ecb/ecb.xsd')
def estado_de_cuenta_combustible2(col, data):
    col.add_map('ecc', 'http://www.sat.gob.mx/ecc')
    col.add_schema('http://www.sat.gob.mx/ecc http://www.sat.gob.mx/sitio_internet/cfd/ecc/ecc.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ecc/ecc.xsd')
def inst_educativas0(col, data):
    col.add_map('iedu', 'http://www.sat.gob.mx/iedu')
    col.add_schema('http://www.sat.gob.mx/iedu http://www.sat.gob.mx/sitio_internet/cfd/iedu/iedu.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/iedu/iedu.xsd')
def impuestos_locales0(col, data):
    col.add_map('implocal', 'http://www.sat.gob.mx/implocal')
    col.add_schema('http://www.sat.gob.mx/implocal http://www.sat.gob.mx/sitio_internet/cfd/implocal/implocal.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/implocal/implocal.xsd')
def ine0(col, data):
    col.add_map('ine', 'http://www.sat.gob.mx/ine')
    col.add_schema('http://www.sat.gob.mx/ine http://www.sat.gob.mx/sitio_internet/cfd/ine/ine10.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ine/ine10.xsd')
def ine1(col, data):
    col.add_map('ine', 'http://www.sat.gob.mx/ine')
    col.add_schema('http://www.sat.gob.mx/ine http://www.sat.gob.mx/sitio_internet/cfd/ine/ine11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ine/ine11.xsd')
def leyendas_fiscales0(col, data):
    col.add_map('leyendasFisc', 'http://www.sat.gob.mx/leyendasFiscales')
    col.add_schema('http://www.sat.gob.mx/leyendasFiscales http://www.sat.gob.mx/sitio_internet/cfd/leyendasFiscales/leyendasFisc.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/leyendasFiscales/leyendasFisc.xsd')
def nomina0(col, data):
    col.add_map('nomina', 'http://www.sat.gob.mx/nomina')
    col.add_schema('http://www.sat.gob.mx/nomina http://www.sat.gob.mx/sitio_internet/cfd/nomina/nomina11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/nomina/nomina11.xsd')
def nomina1(col, data):
    col.add_map('nomina12', 'http://www.sat.gob.mx/nomina12')
    col.add_schema('http://www.sat.gob.mx/nomina12 http://www.sat.gob.mx/sitio_internet/cfd/nomina/nomina12.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/nomina/nomina12.xsd')
def notarios_publicos0(col, data):
    col.add_map('notariospublicos', 'http://www.sat.gob.mx/notariospublicos')
    col.add_schema('http://www.sat.gob.mx/notariospublicos http://www.sat.gob.mx/sitio_internet/cfd/notariospublicos/notariospublicos.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/notariospublicos/notariospublicos.xsd')
def pago_en_especie0(col, data):
    col.add_map('pagoenespecie', 'http://www.sat.gob.mx/pagoenespecie')
    col.add_schema('http://www.sat.gob.mx/pagoenespecie http://www.sat.gob.mx/sitio_internet/cfd/pagoenespecie/pagoenespecie.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/pagoenespecie/pagoenespecie.xsd')
def pfintegrante_coordinado0(col, data):
    col.add_map('pfic', 'http://www.sat.gob.mx/pfic')
    col.add_schema('http://www.sat.gob.mx/pfic http://www.sat.gob.mx/sitio_internet/cfd/pfic/pfic.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/pfic/pfic.xsd')
def prestadores_de_servicios_de_cfd0(col, data):
    col.add_map('psgecfd', 'http://www.sat.gob.mx/psgecfd')
    col.add_schema('http://www.sat.gob.mx/psgecfd http://www.sat.gob.mx/sitio_internet/cfd/psgecfd/psgecfd.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/psgecfd/psgecfd.xsd')
def renovacionysustitucionvehiculos0(col, data):
    col.add_map('decreto', 'http://www.sat.gob.mx/renovacionysustitucionvehiculos')
    col.add_schema('http://www.sat.gob.mx/renovacionysustitucionvehiculos http://www.sat.gob.mx/sitio_internet/cfd/renovacionysustitucionvehiculos/renovacionysustitucionvehiculos.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/renovacionysustitucionvehiculos/renovacionysustitucionvehiculos.xsd')
def parcialesconstruccion0(col, data):
    col.add_map('servicioparcial', 'http://www.sat.gob.mx/servicioparcialconstruccion')
    col.add_schema('http://www.sat.gob.mx/servicioparcialconstruccion http://www.sat.gob.mx/sitio_internet/cfd/servicioparcialconstruccion/servicioparcialconstruccion.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/servicioparcialconstruccion/servicioparcialconstruccion.xsd')
def complemento_spei0(col, data):
    col.add_map('spei', 'http://www.sat.gob.mx/spei')
    col.add_schema('http://www.sat.gob.mx/spei http://www.sat.gob.mx/sitio_internet/cfd/spei/spei.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/spei/spei.xsd')
def por_cuentade_terceros0(col, data):
    col.add_map('terceros', 'http://www.sat.gob.mx/terceros')
    col.add_schema('http://www.sat.gob.mx/terceros http://www.sat.gob.mx/sitio_internet/cfd/terceros/terceros11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/terceros/terceros11.xsd')
def vales_de_despensa0(col, data):
    col.add_map('valesdedespensa', 'http://www.sat.gob.mx/valesdedespensa')
    col.add_schema('http://www.sat.gob.mx/valesdedespensa http://www.sat.gob.mx/sitio_internet/cfd/valesdedespensa/valesdedespensa.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/valesdedespensa/valesdedespensa.xsd')
def vehiculo_usado0(col, data):
    col.add_map('vehiculousado', 'http://www.sat.gob.mx/vehiculousado')
    col.add_schema('http://www.sat.gob.mx/vehiculousado http://www.sat.gob.mx/sitio_internet/cfd/vehiculousado/vehiculousado.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/vehiculousado/vehiculousado.xsd')
def venta_vehiculos0(col, data):
    col.add_map('ventavehiculos', 'http://www.sat.gob.mx/ventavehiculos')
    col.add_schema('http://www.sat.gob.mx/ventavehiculos http://www.sat.gob.mx/sitio_internet/cfd/ventavehiculos/ventavehiculos11.xsd')
    col.add_base('www.sat.gob.mx/sitio_internet/cfd/ventavehiculos/ventavehiculos11.xsd')
def s_cancelacion0(col, data):
    cancelacion0(col, data)
def s_cancelacion1(col, data):
    cancelacion1(col, data)
def s_solicitud_aceptacion_rechazo0(col, data):
    solicitud_aceptacion_rechazo0(col, data)
def s_diverza0(col, data):
    if data.get('Version') == '1.1':
        diverza0(col, data)
def s_archivo0(col, data):
    archivo_type0(col, data)
def s_archivo1(col, data):
    archivo_type1(col, data)
def s_archivo2(col, data):
    archivo_type2(col, data)
def s_archivo3(col, data):
    archivo_type3(col, data)
def s_archivo4(col, data):
    archivo_type4(col, data)
def s_archivo5(col, data):
    archivo_type5(col, data)
def s_archivo6(col, data):
    archivo_type6(col, data)
def s_archivo7(col, data):
    archivo_type7(col, data)
def s_archivo8(col, data):
    archivo_type8(col, data)
def s_archivo9(col, data):
    archivo_type9(col, data)
def s_archivoa(col, data):
    archivo_typea(col, data)
def s_archivob(col, data):
    archivo_typeb(col, data)
def s_archivoc(col, data):
    archivo_typec(col, data)
def s_archivod(col, data):
    archivo_typed(col, data)
def s_archivoe(col, data):
    archivo_typee(col, data)
def s_archivof(col, data):
    archivo_typef(col, data)
def s_archivo10(col, data):
    archivo_type10(col, data)
def s_archivo11(col, data):
    archivo_type11(col, data)
def s_archivo12(col, data):
    archivo_type12(col, data)
def s_auxiliar_ctas0(col, data):
    if data.get('Version') == '1.1':
        auxiliar_ctas0(col, data)
def s_rep_aux_fol0(col, data):
    if data.get('Version') == '1.2':
        rep_aux_fol0(col, data)
def s_balanza0(col, data):
    if data.get('Version') == '1.1':
        balanza0(col, data)
def s_catalogo0(col, data):
    if data.get('Version') == '1.1':
        catalogo0(col, data)
def s_polizas0(col, data):
    if data.get('Version') == '1.1':
        polizas0(col, data)
def s_auxiliar_ctas1(col, data):
    if data.get('Version') == '1.3':
        auxiliar_ctas1(col, data)
def s_rep_aux_fol1(col, data):
    if data.get('Version') == '1.3':
        rep_aux_fol1(col, data)
def s_balanza1(col, data):
    if data.get('Version') == '1.3':
        balanza1(col, data)
def s_catalogo1(col, data):
    if data.get('Version') == '1.3':
        catalogo1(col, data)
def s_polizas1(col, data):
    if data.get('Version') == '1.3':
        polizas1(col, data)
def s_sello_digital_cont_elec0(col, data):
    if data.get('Version') == '1.1':
        sello_digital_cont_elec0(col, data)
def s_servicios_plataformas_tecnologicas0(col, data):
    if data.get('Version') == '1.0':
        servicios_plataformas_tecnologicas0(col, data)
def s_arrendamientoenfideicomiso0(col, data):
    if data.get('Version') == '1.0':
        arrendamientoenfideicomiso0(col, data)
def s_dividendos0(col, data):
    if data.get('Version') == '1.0':
        dividendos0(col, data)
def s_enajenacionde_acciones0(col, data):
    if data.get('Version') == '1.0':
        enajenacionde_acciones0(col, data)
def s_fideicomisonoempresarial0(col, data):
    if data.get('Version') == '1.0':
        fideicomisonoempresarial0(col, data)
def s_intereses0(col, data):
    if data.get('Version') == '1.0':
        intereses0(col, data)
def s_intereseshipotecarios0(col, data):
    if data.get('Version') == '1.0':
        intereseshipotecarios0(col, data)
def s_operacionesconderivados0(col, data):
    if data.get('Version') == '1.0':
        operacionesconderivados0(col, data)
def s_pagosaextranjeros0(col, data):
    if data.get('Version') == '1.0':
        pagosaextranjeros0(col, data)
def s_planesderetiro0(col, data):
    if data.get('Version') == '1.1':
        planesderetiro0(col, data)
def s_planesderetiro1(col, data):
    if data.get('Version') == '1.0':
        planesderetiro1(col, data)
def s_premios0(col, data):
    if data.get('Version') == '1.0':
        premios0(col, data)
def s_retenciones0(col, data):
    if data.get('Version') == '1.0':
        retenciones0(col, data)
def s_sector_financiero0(col, data):
    if data.get('Version') == '1.0':
        sector_financiero0(col, data)
def s_retenciones1(col, data):
    if data.get('Version') == '2.0':
        retenciones1(col, data)
def s_comprobante0(col, data):
    if data.get('Version') == '3.2':
        comprobante0(col, data)
    if data.get('Version') == '3.3':
        comprobante1(col, data)
def s_comprobante1(col, data):
    if data.get('Version') == '4.0':
        comprobante2(col, data)
def s_carta_porte0(col, data):
    if data.get('Version') == '1.0':
        carta_porte0(col, data)
def s_carta_porte1(col, data):
    if data.get('Version') == '2.0':
        carta_porte1(col, data)
def s_comercio_exterior0(col, data):
    if data.get('Version') == '1.1':
        comercio_exterior0(col, data)
def s_comercio_exterior1(col, data):
    if data.get('Version') == '1.0':
        comercio_exterior1(col, data)
def s_estado_de_cuenta_combustible0(col, data):
    if data.get('Version') == '1.1' and data.get('TipoOperacion') == 'Tarjeta':
        estado_de_cuenta_combustible0(col, data)
def s_estado_de_cuenta_combustible1(col, data):
    if data.get('Version') == '1.2' and data.get('TipoOperacion') == 'Tarjeta':
        estado_de_cuenta_combustible1(col, data)
def s_gastos_hidrocarburos0(col, data):
    if data.get('Version') == '1.0':
        gastos_hidrocarburos0(col, data)
def s_ingresos_hidrocarburos0(col, data):
    if data.get('Version') == '1.0':
        ingresos_hidrocarburos0(col, data)
def s_pagos0(col, data):
    if data.get('Version') == '1.0':
        pagos0(col, data)
def s_pagos1(col, data):
    if data.get('Version') == '2.0':
        pagos1(col, data)
def s_timbre_fiscal_digital0(col, data):
    if data.get('Version') == '1.0':
        timbre_fiscal_digital0(col, data)
    if data.get('Version') == '1.1':
        timbre_fiscal_digital1(col, data)
def s_turista_pasajero_extranjero0(col, data):
    if data.get('Version') == '1.0':
        turista_pasajero_extranjero0(col, data)
def s_acreditamiento_ieps0(col, data):
    if data.get('Version') == '1.0':
        acreditamiento_ieps0(col, data)
def s_aerolineas0(col, data):
    if data.get('Version') == '1.0':
        aerolineas0(col, data)
def s_obrasarteantiguedades0(col, data):
    if data.get('Version') == '1.0':
        obrasarteantiguedades0(col, data)
def s_certificadodedestruccion0(col, data):
    if data.get('Version') == '1.0':
        certificadodedestruccion0(col, data)
def s_cfdiregistro_fiscal0(col, data):
    if data.get('Version') == '1.0':
        cfdiregistro_fiscal0(col, data)
def s_consumo_de_combustibles0(col, data):
    if data.get('Version') == '1.1' and data.get('TipoOperacion') == 'monedero electrónico':
        consumo_de_combustibles0(col, data)
def s_consumo_de_combustibles1(col, data):
    if data.get('Version') == '1.0' and data.get('TipoOperacion') == 'monedero electrónico':
        consumo_de_combustibles1(col, data)
def s_detallista0(col, data):
    if data.get('DocumentStructureVersion') == 'AMC8.1':
        detallista0(col, data)
def s_divisas0(col, data):
    if data.get('Version') == '1.0':
        divisas0(col, data)
def s_donatarias0(col, data):
    if data.get('Version') == '1.1':
        donatarias0(col, data)
def s_estado_de_cuenta_bancario0(col, data):
    if data.get('Version') == '1.0':
        estado_de_cuenta_bancario0(col, data)
def s_estado_de_cuenta_combustible2(col, data):
    if data.get('TipoOperacion') == 'Tarjeta':
        estado_de_cuenta_combustible2(col, data)
def s_inst_educativas0(col, data):
    if data.get('Version') == '1.0':
        inst_educativas0(col, data)
def s_impuestos_locales0(col, data):
    if data.get('Version') == '1.0':
        impuestos_locales0(col, data)
def s_ine0(col, data):
    if data.get('Version') == '1.0':
        ine0(col, data)
    if data.get('Version') == '1.1':
        ine1(col, data)
def s_leyendas_fiscales0(col, data):
    if data.get('Version') == '1.0':
        leyendas_fiscales0(col, data)
def s_nomina0(col, data):
    if data.get('Version') == '1.1':
        nomina0(col, data)
def s_nomina1(col, data):
    if data.get('Version') == '1.2':
        nomina1(col, data)
def s_notarios_publicos0(col, data):
    if data.get('Version') == '1.0':
        notarios_publicos0(col, data)
def s_pago_en_especie0(col, data):
    if data.get('Version') == '1.0':
        pago_en_especie0(col, data)
def s_pfintegrante_coordinado0(col, data):
    if data.get('Version') == '1.0':
        pfintegrante_coordinado0(col, data)
def s_prestadores_de_servicios_de_cfd0(col, data):
    prestadores_de_servicios_de_cfd0(col, data)
def s_renovacionysustitucionvehiculos0(col, data):
    if data.get('Version') == '1.0':
        renovacionysustitucionvehiculos0(col, data)
def s_parcialesconstruccion0(col, data):
    if data.get('Version') == '1.0':
        parcialesconstruccion0(col, data)
def s_complemento_spei0(col, data):
    complemento_spei0(col, data)
def s_por_cuentade_terceros0(col, data):
    if data.get('Version') == '1.1':
        por_cuentade_terceros0(col, data)
def s_vales_de_despensa0(col, data):
    if data.get('Version') == '1.0' and data.get('TipoOperacion') == 'monedero electrónico':
        vales_de_despensa0(col, data)
def s_vehiculo_usado0(col, data):
    if data.get('Version') == '1.0':
        vehiculo_usado0(col, data)
def s_venta_vehiculos0(col, data):
    if data.get('Version') == '1.1':
        venta_vehiculos0(col, data)
def s_signature0(col, data):
    signature_type0(col, data)
def s_signature_value0(col, data):
    signature_value_type0(col, data)
def s_signed_info0(col, data):
    signed_info_type0(col, data)
def s_canonicalization_method0(col, data):
    canonicalization_method_type0(col, data)
def s_signature_method0(col, data):
    signature_method_type0(col, data)
def s_reference0(col, data):
    reference_type0(col, data)
def s_transforms0(col, data):
    transforms_type0(col, data)
def s_transform0(col, data):
    transform_type0(col, data)
def s_digest_method0(col, data):
    digest_method_type0(col, data)
def s_digest_value0(col, data):
    pass
def s_key_info0(col, data):
    key_info_type0(col, data)
def s_key_name0(col, data):
    pass
def s_mgmt_data0(col, data):
    pass
def s_key_value0(col, data):
    key_value_type0(col, data)
def s_retrieval_method0(col, data):
    retrieval_method_type0(col, data)
def s_x509data0(col, data):
    x509data_type0(col, data)
def s_pgpdata0(col, data):
    pgpdata_type0(col, data)
def s_spkid_ata0(col, data):
    spkid_ata_type0(col, data)
def s_object0(col, data):
    object_type0(col, data)
def s_manifest0(col, data):
    manifest_type0(col, data)
def s_signature_properties0(col, data):
    signature_properties_type0(col, data)
def s_signature_property0(col, data):
    signature_property_type0(col, data)
def s_dsakey_value0(col, data):
    dsakey_value_type0(col, data)
def s_rsakey_value0(col, data):
    rsakey_value_type0(col, data)
cfdi_schemas = {
    '{http://cancelacfd.sat.gob.mx}Cancelacion': s_cancelacion0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1}Cancelacion': s_cancelacion1,
    '{http://cancelacfd.sat.gob.mx}SolicitudAceptacionRechazo': s_solicitud_aceptacion_rechazo0,
    '{http://www.diverza.com/ns/addenda/diverza/1}diverza': s_diverza0,
    '{http://www.uif.shcp.gob.mx/recepcion/ari}archivo': s_archivo0,
    '{http://www.uif.shcp.gob.mx/recepcion/avi}archivo': s_archivo1,
    '{http://www.uif.shcp.gob.mx/recepcion/bli}archivo': s_archivo2,
    '{http://www.uif.shcp.gob.mx/recepcion/chv}archivo': s_archivo3,
    '{http://www.uif.shcp.gob.mx/recepcion/din}archivo': s_archivo4,
    '{http://www.uif.shcp.gob.mx/recepcion/don}archivo': s_archivo5,
    '{http://www.uif.shcp.gob.mx/recepcion/fep}archivo': s_archivo6,
    '{http://www.uif.shcp.gob.mx/recepcion/fes}archivo': s_archivo7,
    '{http://www.uif.shcp.gob.mx/recepcion/inm}archivo': s_archivo8,
    '{http://www.uif.shcp.gob.mx/recepcion/jys}archivo': s_archivo9,
    '{http://www.uif.shcp.gob.mx/recepcion/mjr}archivo': s_archivoa,
    '{http://www.uif.shcp.gob.mx/recepcion/mpc}archivo': s_archivob,
    '{http://www.uif.shcp.gob.mx/recepcion/oba}archivo': s_archivoc,
    '{http://www.uif.shcp.gob.mx/recepcion/spr}archivo': s_archivod,
    '{http://www.uif.shcp.gob.mx/recepcion/tcv}archivo': s_archivoe,
    '{http://www.uif.shcp.gob.mx/recepcion/tdr}archivo': s_archivof,
    '{http://www.uif.shcp.gob.mx/recepcion/tpp}archivo': s_archivo10,
    '{http://www.uif.shcp.gob.mx/recepcion/tsc}archivo': s_archivo11,
    '{http://www.uif.shcp.gob.mx/recepcion/veh}archivo': s_archivo12,
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas}AuxiliarCtas': s_auxiliar_ctas0,
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios}RepAuxFol': s_rep_aux_fol0,
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion}Balanza': s_balanza0,
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas}Catalogo': s_catalogo0,
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo}Polizas': s_polizas0,
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas}AuxiliarCtas': s_auxiliar_ctas1,
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios}RepAuxFol': s_rep_aux_fol1,
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion}Balanza': s_balanza1,
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas}Catalogo': s_catalogo1,
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo}Polizas': s_polizas1,
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/SelloDigitalContElec}SelloDigitalContElec': s_sello_digital_cont_elec0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10}ServiciosPlataformasTecnologicas': s_servicios_plataformas_tecnologicas0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/arrendamientoenfideicomiso}Arrendamientoenfideicomiso': s_arrendamientoenfideicomiso0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/dividendos}Dividendos': s_dividendos0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/enajenaciondeacciones}EnajenaciondeAcciones': s_enajenacionde_acciones0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial}Fideicomisonoempresarial': s_fideicomisonoempresarial0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/intereses}Intereses': s_intereses0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/intereseshipotecarios}Intereseshipotecarios': s_intereseshipotecarios0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados}Operacionesconderivados': s_operacionesconderivados0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros}Pagosaextranjeros': s_pagosaextranjeros0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro11}Planesderetiro': s_planesderetiro0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro}Planesderetiro': s_planesderetiro1,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/premios}Premios': s_premios0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1}Retenciones': s_retenciones0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/1/sectorfinanciero}SectorFinanciero': s_sector_financiero0,
    '{http://www.sat.gob.mx/esquemas/retencionpago/2}Retenciones': s_retenciones1,
    '{http://www.sat.gob.mx/cfd/3}Comprobante': s_comprobante0,
    '{http://www.sat.gob.mx/cfd/4}Comprobante': s_comprobante1,
    '{http://www.sat.gob.mx/CartaPorte}CartaPorte': s_carta_porte0,
    '{http://www.sat.gob.mx/CartaPorte20}CartaPorte': s_carta_porte1,
    '{http://www.sat.gob.mx/ComercioExterior11}ComercioExterior': s_comercio_exterior0,
    '{http://www.sat.gob.mx/ComercioExterior}ComercioExterior': s_comercio_exterior1,
    '{http://www.sat.gob.mx/EstadoDeCuentaCombustible}EstadoDeCuentaCombustible': s_estado_de_cuenta_combustible0,
    '{http://www.sat.gob.mx/EstadoDeCuentaCombustible12}EstadoDeCuentaCombustible': s_estado_de_cuenta_combustible1,
    '{http://www.sat.gob.mx/GastosHidrocarburos10}GastosHidrocarburos': s_gastos_hidrocarburos0,
    '{http://www.sat.gob.mx/IngresosHidrocarburos10}IngresosHidrocarburos': s_ingresos_hidrocarburos0,
    '{http://www.sat.gob.mx/Pagos}Pagos': s_pagos0,
    '{http://www.sat.gob.mx/Pagos20}Pagos': s_pagos1,
    '{http://www.sat.gob.mx/TimbreFiscalDigital}TimbreFiscalDigital': s_timbre_fiscal_digital0,
    '{http://www.sat.gob.mx/TuristaPasajeroExtranjero}TuristaPasajeroExtranjero': s_turista_pasajero_extranjero0,
    '{http://www.sat.gob.mx/acreditamiento}acreditamientoIEPS': s_acreditamiento_ieps0,
    '{http://www.sat.gob.mx/aerolineas}Aerolineas': s_aerolineas0,
    '{http://www.sat.gob.mx/arteantiguedades}obrasarteantiguedades': s_obrasarteantiguedades0,
    '{http://www.sat.gob.mx/certificadodestruccion}certificadodedestruccion': s_certificadodedestruccion0,
    '{http://www.sat.gob.mx/registrofiscal}CFDIRegistroFiscal': s_cfdiregistro_fiscal0,
    '{http://www.sat.gob.mx/ConsumoDeCombustibles11}ConsumoDeCombustibles': s_consumo_de_combustibles0,
    '{http://www.sat.gob.mx/consumodecombustibles}ConsumoDeCombustibles': s_consumo_de_combustibles1,
    '{http://www.sat.gob.mx/detallista}detallista': s_detallista0,
    '{http://www.sat.gob.mx/divisas}Divisas': s_divisas0,
    '{http://www.sat.gob.mx/donat}Donatarias': s_donatarias0,
    '{http://www.sat.gob.mx/ecb}EstadoDeCuentaBancario': s_estado_de_cuenta_bancario0,
    '{http://www.sat.gob.mx/ecc}EstadoDeCuentaCombustible': s_estado_de_cuenta_combustible2,
    '{http://www.sat.gob.mx/iedu}instEducativas': s_inst_educativas0,
    '{http://www.sat.gob.mx/implocal}ImpuestosLocales': s_impuestos_locales0,
    '{http://www.sat.gob.mx/ine}INE': s_ine0,
    '{http://www.sat.gob.mx/leyendasFiscales}LeyendasFiscales': s_leyendas_fiscales0,
    '{http://www.sat.gob.mx/nomina}Nomina': s_nomina0,
    '{http://www.sat.gob.mx/nomina12}Nomina': s_nomina1,
    '{http://www.sat.gob.mx/notariospublicos}NotariosPublicos': s_notarios_publicos0,
    '{http://www.sat.gob.mx/pagoenespecie}PagoEnEspecie': s_pago_en_especie0,
    '{http://www.sat.gob.mx/pfic}PFintegranteCoordinado': s_pfintegrante_coordinado0,
    '{http://www.sat.gob.mx/psgecfd}PrestadoresDeServiciosDeCFD': s_prestadores_de_servicios_de_cfd0,
    '{http://www.sat.gob.mx/renovacionysustitucionvehiculos}renovacionysustitucionvehiculos': s_renovacionysustitucionvehiculos0,
    '{http://www.sat.gob.mx/servicioparcialconstruccion}parcialesconstruccion': s_parcialesconstruccion0,
    '{http://www.sat.gob.mx/spei}Complemento_SPEI': s_complemento_spei0,
    '{http://www.sat.gob.mx/terceros}PorCuentadeTerceros': s_por_cuentade_terceros0,
    '{http://www.sat.gob.mx/valesdedespensa}ValesDeDespensa': s_vales_de_despensa0,
    '{http://www.sat.gob.mx/vehiculousado}VehiculoUsado': s_vehiculo_usado0,
    '{http://www.sat.gob.mx/ventavehiculos}VentaVehiculos': s_venta_vehiculos0,
    '{http://www.w3.org/2000/09/xmldsig#}Signature': s_signature0,
    '{http://www.w3.org/2000/09/xmldsig#}SignatureValue': s_signature_value0,
    '{http://www.w3.org/2000/09/xmldsig#}SignedInfo': s_signed_info0,
    '{http://www.w3.org/2000/09/xmldsig#}CanonicalizationMethod': s_canonicalization_method0,
    '{http://www.w3.org/2000/09/xmldsig#}SignatureMethod': s_signature_method0,
    '{http://www.w3.org/2000/09/xmldsig#}Reference': s_reference0,
    '{http://www.w3.org/2000/09/xmldsig#}Transforms': s_transforms0,
    '{http://www.w3.org/2000/09/xmldsig#}Transform': s_transform0,
    '{http://www.w3.org/2000/09/xmldsig#}DigestMethod': s_digest_method0,
    '{http://www.w3.org/2000/09/xmldsig#}DigestValue': s_digest_value0,
    '{http://www.w3.org/2000/09/xmldsig#}KeyInfo': s_key_info0,
    '{http://www.w3.org/2000/09/xmldsig#}KeyName': s_key_name0,
    '{http://www.w3.org/2000/09/xmldsig#}MgmtData': s_mgmt_data0,
    '{http://www.w3.org/2000/09/xmldsig#}KeyValue': s_key_value0,
    '{http://www.w3.org/2000/09/xmldsig#}RetrievalMethod': s_retrieval_method0,
    '{http://www.w3.org/2000/09/xmldsig#}X509Data': s_x509data0,
    '{http://www.w3.org/2000/09/xmldsig#}PGPData': s_pgpdata0,
    '{http://www.w3.org/2000/09/xmldsig#}SPKIData': s_spkid_ata0,
    '{http://www.w3.org/2000/09/xmldsig#}Object': s_object0,
    '{http://www.w3.org/2000/09/xmldsig#}Manifest': s_manifest0,
    '{http://www.w3.org/2000/09/xmldsig#}SignatureProperties': s_signature_properties0,
    '{http://www.w3.org/2000/09/xmldsig#}SignatureProperty': s_signature_property0,
    '{http://www.w3.org/2000/09/xmldsig#}DSAKeyValue': s_dsakey_value0,
    '{http://www.w3.org/2000/09/xmldsig#}RSAKeyValue': s_rsakey_value0,
}
